﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace PolyEvalExperiment
{
    class Program
    {
        static void Main(string[] args)
        {
            DateTime before1 = GetTime();
            EvalPolyTest1("input.txt", "output1.txt");
            DateTime after1 = GetTime();
            double diff1 = GetDiff(after1, before1);

            // running my algorithm twice just to make sure there are no disadvantages with program startup and initial file read
            before1 = GetTime();
            EvalPolyTest1("input.txt", "output1.txt");
            after1 = GetTime();
            diff1 = GetDiff(after1, before1);

            DateTime before2 = GetTime();
            EvalPolyTest2("input.txt", "output2.txt");
            DateTime after2 = GetTime();
            double diff2 = GetDiff(after2, before2);

            // getting a baseline time:  just file operations, no polynomial evaluations
            DateTime beforeBase = GetTime();
            BaseLine("input.txt", "output3.txt");
            DateTime afterBase = GetTime();
            double diffBase = GetDiff(afterBase, beforeBase);

            Console.WriteLine("Baseline (file read/write) time: {0} seconds\n", diffBase);
            Console.WriteLine("My Algorithm:  {0} seconds\n", (diff1 - diffBase));
            Console.WriteLine("Your Algorithm {0} seconds\n", (diff2 - diffBase));
            Console.WriteLine("Improvement ratio: {0}\n", (diff2 - diffBase) * 1.0 / (diff1 - diffBase));

            Console.ReadLine();
        }

        // the polynomial evaluation algorithm used by EvalPolyTest1 (this is my algorithm)
        // note that coeffs[i] is the coefficient of x^i
        static double EvalPoly1(double[] coeffs, int degree, double value)
        {
            double P = 0;
            for (int i = 0; i <= degree; i++)
            {
                double term = coeffs[i];
                for (int j = 1; j <= i; j++)
                {
                    term = term * value;
                }
                P = P + term;

            }
            return P;
        }

        static void EvalPolyTest1(String infile, String outfile)
        {
            string[] lines = File.ReadAllLines(infile);
            using (StreamWriter file = new StreamWriter(outfile))
            {
                for (int i = 0; i < lines.Length; i = i + 2)
                {
                    String[] coeffStrs = lines[i].Split();
                    String[] valueStrs = lines[i + 1].Split();
                    int coeffCount = coeffStrs.Length;
                    int valueCount = valueStrs.Length;
                    double[] coeffs = new double[coeffCount];
                    double[] values = new double[valueCount];
                    for (int j = 0; j < coeffCount; j++)
                    {
                        coeffs[j] = double.Parse(coeffStrs[j]);
                    }
                    for (int j = 0; j < valueCount; j++)
                    {
                        values[j] = double.Parse(valueStrs[j]);
                    }
                    for (int j = 0; j < valueCount; j++)
                    {
                        double ans = EvalPoly1(coeffs, coeffCount - 1, values[j]);
                        file.WriteLine("{0:E5}", ans);
                    }
                }
            }
        }

        // baseLine simply reads the file but performs no computations
        public static void BaseLine(String infile, String outfile)
        {
            string[] lines = File.ReadAllLines(infile);
            using (StreamWriter file = new StreamWriter(outfile))
            {
                for (int i = 0; i < lines.Length; i = i + 2)
                {
                    String[] coeffStrs = lines[i].Split();
                    String[] valueStrs = lines[i + 1].Split();
                    int coeffCount = coeffStrs.Length;
                    int valueCount = valueStrs.Length;
                    double[] coeffs = new double[coeffCount];
                    double[] values = new double[valueCount];
                    for (int j = 0; j < coeffCount; j++)
                    {
                        coeffs[j] = double.Parse(coeffStrs[j]);
                    }
                    for (int j = 0; j < valueCount; j++)
                    {
                        values[j] = double.Parse(valueStrs[j]);
                    }
                    for (int j = 0; j < valueCount; j++)
                    {
                        file.WriteLine("0.00000+100");
                    }
                }
            }
        }

        // your polynomial evaluation algorithm used by EvalPolyTest2
        // to be filled up
        // note that coeffs[i] is the coefficient of x^i
        static double EvalPoly2(double[] coeffs, int degree, double value)
        {
            double P = 0;
            // TODO: your algorithm here
            return P;
        }


        static void EvalPolyTest2(String infile, String outfile)
        {
            string[] lines = File.ReadAllLines(infile);
            using (StreamWriter file = new StreamWriter(outfile))
            {
                for (int i = 0; i < lines.Length; i = i + 2)
                {
                    String[] coeffStrs = lines[i].Split();
                    String[] valueStrs = lines[i + 1].Split();
                    int coeffCount = coeffStrs.Length;
                    int valueCount = valueStrs.Length;
                    double[] coeffs = new double[coeffCount];
                    double[] values = new double[valueCount];
                    for (int j = 0; j < coeffCount; j++)
                    {
                        coeffs[j] = double.Parse(coeffStrs[j]);
                    }
                    for (int j = 0; j < valueCount; j++)
                    {
                        values[j] = double.Parse(valueStrs[j]);
                    }
                    for (int j = 0; j < valueCount; j++)
                    {
                        double ans = EvalPoly2(coeffs, coeffCount - 1, values[j]);
                        file.WriteLine("{0:E5}", ans);
                    }
                }
            }
        }

        static DateTime GetTime()
        {
            return DateTime.Now;
            ;
        }

        // number of seconds between two time stamps
        static double GetDiff(DateTime a, DateTime b)
        {
            TimeSpan t = a - b;
            return t.TotalMilliseconds / 1000.0;
        }

    }
}
